@@ -1,5 +1,7 @@
 #!/usr/bin/env python3
 
+from typing import List, Tuple
+
 import re
 from enum import Enum
 from collections import namedtuple, deque
@@ -72,21 +74,22 @@ class StringView:
             v = v[:-len(e.group())]
         return v
 
-    BLANKS = re.compile(r"\s+\Z")
+    BLANKS = re.compile(r"\s*\Z")
     def isspace(self):
         return bool(self.match(StringView.BLANKS))
 
 class Line(namedtuple("Line", "num parts")):
     def __len__(self):
+        """Compute the number of characters in `self`.
+        >>> len(Line(0, ("a", "bc", "def")))
+        6
+        """
         return sum(len(p) for p in self.parts)
 
     def __str__(self):
         s = "".join(str(p) for p in self.parts)
         return s if not s.isspace() else ""
 
-    def __radd__(self, s):
-        self.parts.append(s)
-
     def isspace(self):
         return all(p.isspace() for p in self.parts)
 
@@ -120,13 +123,10 @@ def replace(line, pairs):
         line = line.replace(src, dst)
     return line
 
-def blank(line):
-    return (not line) or line.isspace()
-
 def strip_block(lines, beg, end):
-    while beg < len(lines) and blank(lines[beg]):
+    while beg < len(lines) and lines[beg].isspace():
         beg += 1
-    while end > beg and blank(lines[end - 1]):
+    while end > beg and lines[end - 1].isspace():
         end -= 1
     return (beg, end)
 
@@ -274,7 +274,7 @@ def coq_partition(doc):
     """
     pos = 0
     spans = []
-    stack = [(0, 0, State.CODE)]
+    stack: List[Tuple[int, int, State]] = [(0, 0, State.CODE)]
     spans = []
     while True:
         start, token_end, state = stack[-1]
@@ -338,21 +338,73 @@ COQDOC_OPEN = re.compile(r"[(][*][*]\s[ \t]*")
 COQDOC_CLOSE = re.compile(r"[ \t]*[*]+[)]\Z")
 
 DEFAULT_HEADER = ".. coq::"
-DIRECTIVE = re.compile(r"([ \t]*)([.][.] coq::.*)?")
+INDENT = re.compile(r"(?P<indent>[ ]*)")
+COQ_DIRECTIVE = re.compile(r"(?P<indent>[ \t]*)([.][.] coq::.*)")
 
-Lit = namedtuple("Lit", "lines directive indent")
+Lit = namedtuple("Lit", "lines directive_lines indent")
 CodeBlock = namedtuple("CodeBlock", "lines indent")
 
+def _last_coq_directive(lines):
+    r"""Scan backwards across `lines` to find the beginning of the Coq directive.
+
+    >>> _, ls = number_lines(StringView('''\
+    ... Text.
+    ... .. coq:: unfold
+    ...    :name: nm
+    ... '''), 6)
+    >>> _last_coq_directive(ls) # doctest: +ELLIPSIS
+    (...[Line(num=6, parts=['Text.'])]...,
+        <re.Match ...'.. coq:: unfold'>,
+     ...[Line(num=7, parts=['.. coq:: unfold']),
+         Line(num=8, parts=['   :name: nm']),
+         Line(num=9, parts=[''])]...)
+
+    >>> _, ls = number_lines(StringView('''\
+    ... Text.
+    ...    .. coq:: unfold
+    ...    Text.
+    ... '''), 6)
+    >>> _last_coq_directive(ls) # doctest: +ELLIPSIS
+    (...[Line(num=6, parts=['Text.']),
+         Line(num=7, parts=['   .. coq:: unfold']),
+         Line(num=8, parts=['   Text.']),
+         Line(num=9, parts=[''])]...)
+
+    >>> _, ls = number_lines(StringView('Text.\n   Text.'), 6)
+    >>> _last_coq_directive(ls) # doctest: +ELLIPSIS
+    (...[Line(num=6, parts=['Text.']),
+         Line(num=7, parts=['   Text.'])]...)
+
+    """
+    directive = deque()
+    expected_coq_indent = float("+inf")
+    while lines:
+        line = lines.pop()
+        directive.appendleft(line)
+        indent = measure_indentation(line)
+        m = line.match(COQ_DIRECTIVE)
+        if m:
+            if indent <= expected_coq_indent:
+                return lines, m, directive
+            break # pragma: no cover # coverage.py bug
+        if not line.isspace():
+            expected_coq_indent = min(expected_coq_indent, indent - 3)
+            if expected_coq_indent < 0:
+                break # No need to keep looping
+    lines.extend(directive)
+    return lines, None, None
+
 def lit(lines, indent):
     strip_deque(lines)
-    m = lines and lines[-1].match(DIRECTIVE)
-    indent, directive = m.groups() if m else (indent, None)
-    if directive:
-        directive = lines.pop()
+    lines, m, directive_lines = _last_coq_directive(lines)
+    if directive_lines:
+        indent = m.group("indent")
         strip_deque(lines)
     else:
-        directive = indent + DEFAULT_HEADER
-    return Lit(lines, directive=directive, indent=indent)
+        if lines:
+            indent = lines[-1].match(INDENT).group("indent")
+        directive_lines = [indent + DEFAULT_HEADER]
+    return Lit(lines, directive_lines=directive_lines, indent=indent)
 
 def number_lines(span, start):
     lines = span.split("\n")
@@ -360,12 +412,12 @@ def number_lines(span, start):
     return start + len(lines) - 1, d
 
 def gen_rst(spans):
-    linum, indent, prefix = 0, "", DEFAULT_HEADER
+    linum, indent, prefix = 0, "", [DEFAULT_HEADER]
     for span in spans:
         if isinstance(span, Comment):
             linum, lines = number_lines(span.v.trim(LIT_OPEN, LIT_CLOSE), linum)
             litspan = lit(lines, indent)
-            indent, prefix = litspan.indent, litspan.directive
+            indent, prefix = litspan.indent, litspan.directive_lines
             if litspan.lines:
                 yield from (replace(l, UNQUOTE_PAIRS) for l in litspan.lines)
                 yield ""
@@ -375,7 +427,7 @@ def gen_rst(spans):
             for l in lines:
                 l.parts.insert(0, indent + "   ")
             if lines:
-                yield prefix
+                yield from prefix
                 yield ""
                 yield from lines
                 yield ""
@@ -399,6 +451,40 @@ def coq2rst_lines(coq):
     return gen_rst(coq_partition_literate(coq))
 
 def coq2rst(coq):
+    """Translate a fragment of Coq code to reST.
+
+    >>> print(coq2rst('''
+    ... (*|
+    ... Example:
+    ... |*)
+    ...
+    ... Goal True.
+    ...
+    ... (*|
+    ... Second example:
+    ...
+    ... .. coq::
+    ...    :name:
+    ...       snd
+    ... |*)
+    ...
+    ... exact I. Qed.
+    ... '''))
+    Example:
+    <BLANKLINE>
+    .. coq::
+    <BLANKLINE>
+       Goal True.
+    <BLANKLINE>
+    Second example:
+    <BLANKLINE>
+    .. coq::
+       :name:
+          snd
+    <BLANKLINE>
+       exact I. Qed.
+    <BLANKLINE>
+    """
     return join_lines(coq2rst_lines(coq))
 
 def coq2rst_marked(coq, point, marker):
@@ -417,31 +503,57 @@ def coq2rst_marked(coq, point, marker):
 # note that it detects *all* ‘.. coq::’ blocks, including quoted ones.
 
 COQ_BLOCK = re.compile(r"""
-^(?P<indent>[ \t]*)[.][.][ ]coq::.*
+(?P<directive>
+ ^(?P<indent>[ ]*)
+  [.][.][ ]coq::.*
+  (?P<options>
+   (?:\n
+     (?P=indent)[ ][ ][ ] [ \t]*[^ \t].*$)*))
 (?P<body>
- (?:\n
-    (?:[ \t]*\n)*
-    (?P=indent)[ ][ ][ ].*$)*)
+   (?:\n
+     (?:[ \t]*\n)*
+     (?P=indent)[ ][ ][ ] .*$)*)
 """, re.VERBOSE | re.MULTILINE)
 
 def rst_partition(s):
+    """Identify ``.. coq::`` blocks in reST sources.
+
+    >>> print(list(rst_partition('''\\
+    ... .. coq::
+    ...
+    ...      Goal True.
+    ...        exact I. Qed.\\
+    ... ''')))
+    [Lit(lines=deque([Line(num=0, parts=[''])]),
+         directive_lines=deque([Line(num=0, parts=['.. coq::'])]),
+         indent=0),
+     CodeBlock(lines=deque([Line(num=0, parts=['']),
+                            Line(num=1, parts=['']),
+                            Line(num=2, parts=['  Goal True.']),
+                            Line(num=3, parts=['    exact I. Qed.'])]),
+               indent=0)]
+    <BLANKLINE>
+    """
     beg, linum = 0, 0
     for m in COQ_BLOCK.finditer(s):
         indent = len(m.group("indent"))
         rst = StringView(s, beg, m.start())
-        block = StringView(s, *m.span())
+        directive = StringView(s, *m.span('directive'))
+        body = StringView(s, *m.span('body'))
 
         linum, rst_lines = number_lines(rst, linum)
-        linum, block_lines = number_lines(block, linum)
-        directive = block_lines.popleft()
+        linum, directive_lines = number_lines(directive, linum)
+        linum, body_lines = number_lines(body, linum)
 
-        yield Lit(rst_lines, directive=directive, indent=indent)
-        yield CodeBlock(block_lines, indent=indent)
+        # body_lines.popleft() # Discard initial blank
+
+        yield Lit(rst_lines, directive_lines=directive_lines, indent=indent)
+        yield CodeBlock(body_lines, indent=indent)
         beg = m.end()
     if beg < len(s):
         rst = StringView(s, beg, len(s))
         linum, lines = number_lines(rst, linum)
-        yield Lit(lines, directive=None, indent=None)
+        yield Lit(lines, directive_lines=None, indent=None)
 
 # Conversion
 # ----------
@@ -451,29 +563,34 @@ def measure_indentation(line):
     m = line.match(INDENTATION_RE)
     return m.end() - m.start()
 
+def redundant_directive(directive_lines, directive_indent, last_indent):
+    return (
+        directive_lines and
+        len(directive_lines) == 1 and
+        str(directive_lines[0]).strip() == DEFAULT_HEADER
+        and directive_indent == last_indent
+    )
+
 def trim_rst_block(block, last_indent, keep_empty):
     strip_deque(block.lines)
-    directive_indent = block.indent # Stored here for convenience
     last_indent = measure_indentation(block.lines[-1]) if block.lines else last_indent
 
-    directive = block.directive
-    keep_empty = keep_empty and directive is not None
-    if (directive
-        and str(directive).strip() == DEFAULT_HEADER
-        and directive_indent == last_indent):
-        directive = None
+    directive_lines = block.directive_lines
+    keep_empty = keep_empty and directive_lines
+    if redundant_directive(directive_lines, block.indent, last_indent):
+        directive_lines = None
 
-    if not block.lines and not directive:
+    if not block.lines and not directive_lines:
         if keep_empty:
             yield "(*||*)"
             yield ""
     else:
         yield "(*|"
         yield from (replace(l, QUOTE_PAIRS) for l in block.lines)
-        if directive:
+        if directive_lines:
             if block.lines:
                 yield ""
-            yield directive
+            yield from directive_lines
         yield "|*)"
         yield ""
 
@@ -497,6 +614,40 @@ def rst2coq_lines(rst):
     return gen_coq(rst_partition(rst))
 
 def rst2coq(rst):
+    """Translate a fragment of reST code to Coq.
+
+    >>> print(rst2coq('''
+    ... Example:
+    ...
+    ... .. coq::
+    ...
+    ...    Goal True.
+    ...
+    ... Second example:
+    ...
+    ... .. coq::
+    ...    :name:
+    ...       snd
+    ...
+    ...    exact I. Qed.
+    ... '''))
+    (*|
+    Example:
+    |*)
+    <BLANKLINE>
+    Goal True.
+    <BLANKLINE>
+    (*|
+    Second example:
+    <BLANKLINE>
+    .. coq::
+       :name:
+          snd
+    |*)
+    <BLANKLINE>
+    exact I. Qed.
+    <BLANKLINE>
+    """
     return join_lines(rst2coq_lines(rst))
 
 def rst2coq_marked(rst, point, marker):
@@ -545,10 +696,5 @@ def main():
             contents = fstream.read()
     sys.stdout.write(args.fn(contents))
 
-def run_doctest():
-    import sys
-    import doctest
-    doctest.debug(sys.modules.get('__main__'), "__main__.partition", pm=True)
-
 if __name__ == '__main__':
     main()
