@@ -28,41 +28,68 @@ Invoke with ``python3 -m alectryon.minimal --help``.
 
 # pylint: disable=dangerous-default-value
 
-from docutils.parsers.rst import directives, roles, Directive
-import docutils.parsers.rst.directives.body # pylint: disable=unused-import
+from typing import Dict, Any
+
+from docutils import nodes
+from docutils.parsers.rst import directives, roles, Directive # type: ignore
+import docutils.parsers.rst.directives.body # type: ignore # pylint: disable=unused-import
 
 ## Directives
 
-class CoqDirective(directives.body.CodeBlock):
+class ProverDirective(directives.body.CodeBlock):
     final_argument_whitespace = True
-    option_spec = {}
+    option_spec: Dict[str, Any] = {}
 
     def run(self):
-        self.arguments = ["coq"] # Ignore arguments
+        self.arguments = [self.name] # Ignore arguments
         return super().run()
 
 class NoOp(Directive):
+    has_content = True
     def run(self):
-        pass
+        return []
+
+class Id(Directive):
+    has_content = True
+    def run(self):
+        return [nodes.literal("".join(self.content))]
+
+ALL_LANGUAGES = ["coq"]
 
 # Treat .. coq:: as a regular code block and ignore .. alectryon-toggle::
-DIRECTIVES = {"coq": CoqDirective,
-              "alectryon-toggle": NoOp}
+DIRECTIVES = {**{lang: ProverDirective for lang in ALL_LANGUAGES},
+              "alectryon-toggle": NoOp,
+              "exercise": Id,
+              "directive": NoOp,
+              "massert": NoOp,
+              "mquote": Id}
 
 ## Map :coq: to plain literals
 
-def coq_code_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
-    options = {**options.copy(), "language": "coq", "classes": "highlight"}
-    return roles.code_role(role, rawtext, text, lineno, inliner, options, content)
+def custom_code_role(lang):
+    def _code_role(role, rawtext, text, lineno, inliner, options={}, content=[]):
+        options = {**options, "language": lang, "classes": "highlight"}
+        return roles.code_role(role, rawtext, text, lineno, inliner, options, content)
+    return _code_role
 
 def no_op(role, rawtext, text, lineno, inliner, options={}, content=[]):
     return roles.generic_custom_role(
         role, rawtext, text, lineno, inliner, options, content)
-no_op.options = {'url': directives.unchanged}
+no_op.options = { # type: ignore
+    'url': directives.unchanged,
+    'counter-style': directives.unchanged,
+    'prefix': directives.unchanged,
+    'kind': directives.unchanged,
+    'language': directives.unchanged,
+}
 
-ROLES = {"coqid": no_op,
-         "coq": coq_code_role,
-         "alectryon-bubble": no_op}
+ROLES = {
+    **{lang: custom_code_role(lang) for lang in ALL_LANGUAGES},
+    "coqid": no_op,
+    "alectryon-bubble": no_op,
+    "mref": no_op,
+    "mquote": no_op,
+}
 
 def docutils_setup():
     for name, directive in DIRECTIVES.items():
@@ -71,6 +98,15 @@ def docutils_setup():
         roles.register_canonical_role(name, role)
 
 def cli():
+    """Run a minimal Alectryon that skips special roles and directives.
+
+    >>> from . import docutils as ref
+    >>> sorted(set(d.name for d in ref.DIRECTIVES) ^ set(DIRECTIVES))
+    []
+    >>> sorted(set(r.name for r in ref.ROLES) - set(ROLES))
+    []
+    """
+
     import locale
     locale.setlocale(locale.LC_ALL, '')
     from docutils.core import publish_cmdline, default_description
